@@ -6,4 +6,4 @@ from .progress import Progress
 from .warningbar import WarningBar, Indicator
 from .target import Target, TargetList, TargetDialog, TargetSelector
 from .diskmanager import DiskManager
-from .directorysetter import DirectorySetter
\ No newline at end of file
+from .infodialog import InfoDialog
\ No newline at end of file
