@@ -1,5 +1,5 @@
 #/********************************************************************
-# libonvif/onvif-gui/onvif_gui/main.py 
+# onvif-gui/onvif_gui/main.py 
 #
 # Copyright (c) 2023  Stephen Rhodes
 #
@@ -27,32 +27,31 @@ from datetime import datetime
 import importlib.util
 from pathlib import Path
 from PyQt6.QtWidgets import QApplication, QMainWindow, QLabel, QSplitter, \
-    QTabWidget, QMessageBox, QDialog, QGridLayout
+    QTabWidget, QMessageBox, QDialog, QGridLayout, QWidget
 from PyQt6.QtCore import pyqtSignal, QObject, QSettings, QDir, QSize, QTimer, Qt
 from PyQt6.QtGui import QIcon, QGuiApplication, QMovie
 from onvif_gui.panels import CameraPanel, FilePanel, SettingsPanel, VideoPanel, \
-    AudioPanel
-from onvif_gui.enums import ProxyType, Style, PkgType
+    AudioPanel, PicturePanel
+from onvif_gui.enums import ProxyType, Style
 from onvif_gui.glwidget import GLWidget
 from onvif_gui.manager import Manager
 from onvif_gui.player import Player
-from onvif_gui.enums import StreamState
 from onvif_gui.protocols import ServerProtocols, ClientProtocols, ListenProtocols
 from onvif_gui.components.diskmanager import DiskManager
-import avio
 import kankakee
 import platform
 import subprocess
 import requests
 import onvif_gui
 import threading
+import argparse
 
 if sys.platform == "win32":
     from zipfile import ZipFile
 else:
     import tarfile
 
-VERSION = "3.1.10"
+VERSION = "3.3.4"
 
 class TimerSignals(QObject):
     timeoutPlayer = pyqtSignal(str)
@@ -122,6 +121,15 @@ class WaitDialog(QDialog):
 
     def sizeHint(self):
         return QSize(300, 100)
+    
+    def show(self, msg):
+        self.lblMessage.setText(msg)
+        super().show()
+        sleep(0.5)
+
+    def showModal(self, msg):
+        self.lblMessage.setText(msg)
+        self.exec()
 
 class MainWindowSignals(QObject):
     started = pyqtSignal(str)
@@ -138,25 +146,11 @@ class MainWindow(QMainWindow):
     def __init__(self, clear_settings=False, settings_profile="gui", parent_window=None):
         super().__init__()
 
+
         self.version = VERSION
         self.logger_id = logger.add(self.getLogFilename(), rotation="1 MB")
         logger.debug(f'starting onvif-gui version: {VERSION}')
 
-        if sys.platform == "linux":
-            self.pkg_type = PkgType.NATIVE
-            for key in os.environ:
-                if "SNAP" in key:
-                    self.pkg_type = PkgType.SNAP
-                if "FLATPAK" in key:
-                    self.pkg_type = PkgType.FLATPAK
-            match self.pkg_type:
-                case PkgType.NATIVE:
-                    logger.debug("Linux package type is NATIVE")
-                case PkgType.SNAP:
-                    logger.debug("Linux package type is SNAP")
-                case PkgType.FLATPAK:
-                    logger.debug("Linux package type is FLATPAK")
-
         self.settings_profile = settings_profile
         self.parent_window = parent_window
 
@@ -164,13 +158,14 @@ class MainWindow(QMainWindow):
         self.focus_window = None
         self.reader_window = None
         self.external_windows = []
-        self.audioStatus = avio.AudioStatus.UNINITIALIZED
         self.audioLock = False
         self.mediamtx_process = None
+        self.http_process = None
         self.viewer_cameras_filled = False
         self.alarm_ordinals = {}
         self.alarm_states = []
         self.last_alarm = None
+        self.enable_disk_mgmt = False
 
         self.program_name = f'Onvif GUI version {VERSION}'
         self.setWindowTitle(self.program_name)
@@ -217,6 +212,11 @@ class MainWindow(QMainWindow):
         self.filePanel = FilePanel(self)
         self.filePanel.control.setBtnMute()
         self.filePanel.control.setSldVolume()
+        self.picturePanel = PicturePanel(self)
+        if not sys.platform == "win32":
+            self.signals.started.connect(self.picturePanel.onMediaStarted)
+        self.signals.progress.connect(self.picturePanel.onMediaProgress)
+        self.signals.stopped.connect(self.picturePanel.onMediaStopped)
         self.signals.started.connect(self.filePanel.onMediaStarted)
         self.signals.stopped.connect(self.filePanel.onMediaStopped)
         self.signals.progress.connect(self.filePanel.onMediaProgress)
@@ -234,7 +234,9 @@ class MainWindow(QMainWindow):
         hideCameras = bool(int(self.settings.value(self.filePanel.control.hideCameraKey, 0)))
         if not hideCameras:
             self.tab.addTab(self.cameraPanel, "Cameras")
-        self.tab.addTab(self.filePanel, "Files")
+        if settings_profile == "Reader":
+            self.tab.addTab(self.picturePanel, "Pictures")
+            self.tab.addTab(self.filePanel, "Videos")
         if not hideCameras:
             self.tab.addTab(self.settingsPanel, "Settings")
         if self.settingsPanel.proxy.generateAlarmsLocally() and not hideCameras:
@@ -244,8 +246,13 @@ class MainWindow(QMainWindow):
         self.tabIndex = 0
         self.tabBugFix = False
 
+        display = QWidget()
+        lytDisplay = QGridLayout(display)
+        lytDisplay.addWidget(self.glWidget, 0, 0, 1, 1)
+        lytDisplay.setContentsMargins(0, 0, 0, 0)
+
         self.split = QSplitter()
-        self.split.addWidget(self.glWidget)
+        self.split.addWidget(display)
         self.split.addWidget(self.tab)
         self.split.setStretchFactor(0, 10)
         self.split.splitterMoved.connect(self.splitterMoved)
@@ -297,7 +304,10 @@ class MainWindow(QMainWindow):
         if appearance == "Light":
             self.setStyleSheet(self.style(Style.LIGHT))
 
-        collapsed = int(self.settings.value(self.collapsedKey, 0))
+        collapsed_default_value = 0
+        if self.settings_profile == "Focus":
+            collapsed_default_value = 1
+        collapsed = int(self.settings.value(self.collapsedKey, collapsed_default_value))
         if collapsed:
             self.collapseSplitter()
         else:
@@ -309,16 +319,21 @@ class MainWindow(QMainWindow):
         spec = importlib.util.spec_from_file_location("VideoConfigure", workerLocation)
         sys.modules["VideoConfigure"] = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(sys.modules["VideoConfigure"])
+        #sleep(0.5)
         self.signals.hideWaitDialog.emit()
 
     def loadVideoConfigure(self, workerName):
         workerLocation = f'{self.videoPanel.stdLocation}/{workerName}'
-        if sys.platform == "win32" or (sys.platform == "linux" and self.isVisible()) or sys.platform == "darwin":
-            thread = threading.Thread(target=self.specVideo, args=(workerLocation,))
-            thread.start()
-            self.signals.showWaitDialog.emit("Please wait for modules to load")
-        else:
+        if workerName == "motion.py":
+            # weird linux bug that shows dlgWait delayed causing missed signal hideWaitDialog, see sleep in specVideo
             self.specVideo(workerLocation)
+        else:
+            if sys.platform == "win32" or (sys.platform == "linux" and self.isVisible()) or sys.platform == "darwin":
+                thread = threading.Thread(target=self.specVideo, args=(workerLocation,))
+                thread.start()
+                self.signals.showWaitDialog.emit("Please wait for modules to load")
+            else:
+                self.specVideo(workerLocation)
 
         self.videoConfigure = sys.modules["VideoConfigure"].VideoConfigure(self)
         self.cameraPanel.lstCamera.currentItemChanged.connect(self.videoConfigure.setCamera)
@@ -335,15 +350,9 @@ class MainWindow(QMainWindow):
         if self.videoWorkerHook:
             if not self.videoWorker:
                 self.videoWorker = self.videoWorkerHook.VideoWorker(self)
-                if player.isCameraStream():
-                    player.clearCache()
 
         if self.videoWorker:
-            result = self.videoWorker(frame, player)
-            if result is not None:
-                frame = result
-
-        return frame
+            return self.videoWorker(frame, player)
 
     def loadAudioConfigure(self, workerName):
         spec = importlib.util.spec_from_file_location("AudioConfigure", self.audioPanel.stdLocation + "/" + workerName)
@@ -354,8 +363,9 @@ class MainWindow(QMainWindow):
         self.cameraPanel.lstCamera.currentItemChanged.connect(self.audioConfigure.setCamera)
         self.audioPanel.setPanel(self.audioConfigure)
     
-    def pyAudioCallback(self, frame, player):
+    def pyAudioCallback(self, frame, uri):
         try:
+            player = self.pm.getPlayer(uri)
             if player.analyze_audio:
 
                 while self.audioLock:
@@ -386,49 +396,27 @@ class MainWindow(QMainWindow):
         except Exception as ex:
             logger.error(f'Audio callback error: {ex}')
 
-        return frame
-    
     def playMedia(self, uri, file_start_from_seek=-1.0):
-
         if not uri:
             logger.debug(f'Attempt to create player with null uri')
             return
 
-        count = 0
-
-        if self.settings_profile == "Focus" or self.settings_profile == "Reader":
+        if self.settings_profile == "Focus":
             self.closeAllStreams()
-
-        while player := self.pm.getPlayer(uri):
-            sleep(0.01)
-            count += 1
-            if count > 20:
-                logger.debug(f'Duplicate media uri from {self.getCameraName(uri)}:{uri} is blocking launch of new player, requesting shutdown')
-                player.requestShutdown()
-                return
-
+    
         player = Player(uri, self)
-        player.file_start_from_seek = file_start_from_seek
 
-        player.pyAudioCallback = self.pyAudioCallback
-        player.video_filter = "format=rgb24"
-        player.packetDrop = self.packetDrop
-        player.renderCallback = self.glWidget.renderCallback
         player.mediaPlayingStarted = self.mediaPlayingStarted
         player.mediaPlayingStopped = self.mediaPlayingStopped
         player.errorCallback = self.errorCallback
         player.infoCallback = self.infoCallback
-        player.getAudioStatus = self.getAudioStatus
-        player.setAudioStatus = self.setAudioStatus
-        player.hw_device_type = self.settingsPanel.general.getDecoder()
+        player.str_video_filter = "format=rgb24"
+        player.renderCallback = self.glWidget.renderCallback
+        player.str_hw_device_type = self.settingsPanel.general.cmbDecoder.currentText()
         player.audio_driver_index = self.settingsPanel.general.cmbAudioDriver.currentIndex()
 
         if player.isCameraStream():
             if profile := self.cameraPanel.getProfile(uri):
-                player.vpq_size = self.settingsPanel.general.spnCacheMax.value()
-                player.apq_size = self.settingsPanel.general.spnCacheMax.value()
-                if profile.audio_encoding() == "AAC" and profile.audio_sample_rate() and profile.frame_rate():
-                    player.apq_size = int(player.vpq_size * profile.audio_sample_rate() / profile.frame_rate())
                 player.buffer_size_in_seconds = self.settings.value(self.settingsPanel.alarm.bufferSizeKey, 10)
                 player.onvif_frame_rate.num = profile.frame_rate()
                 player.onvif_frame_rate.den = 1
@@ -436,26 +424,35 @@ class MainWindow(QMainWindow):
                 player.disable_video = profile.getDisableVideo()
                 player.hidden = profile.getHidden()
                 if not player.hidden:
+                    player.packetDrop = self.packetDrop
                     player.last_render = datetime.now()
-                player.desired_aspect = profile.getDesiredAspect()
-                player.analyze_video = profile.getAnalyzeVideo()
-                player.analyze_audio = profile.getAnalyzeAudio()
-                player.sync_audio = profile.getSyncAudio()
-                camera = self.cameraPanel.getCamera(uri)
-                if camera:
-                    #player.systemTabSettings() = camera.systemTabSettings
-                    player.setVolume(camera.volume)
-                    player.setMute(camera.mute)
+                    player.pyAudioCallback = self.pyAudioCallback
+                    player.desired_aspect = profile.getDesiredAspect()
+                    player.analyze_video = profile.getAnalyzeVideo()
+                    player.analyze_audio = profile.getAnalyzeAudio()
+                    if camera := self.cameraPanel.getCamera(uri):
+                        player.setVolume(camera.volume)
+                        player.setMute(camera.mute)
             else:
-                logger.error(f'play media profile was not found: {uri}')
+                logger.error(f'{self.settings_profile} play media profile was not found: {uri}')
         else:
+            player.live_stream = False
+            player.file_start_from_seek = file_start_from_seek
             player.request_reconnect = False
             player.setVolume(self.filePanel.getVolume())
             player.setMute(self.filePanel.getMute())
             player.progressCallback = self.mediaProgress
 
+        if not player.isCameraStream():
+            while self.pm.hasFilePlayers():
+                sleep(0.01)
+                
         self.pm.startPlayer(player)
 
+        # this is to prevent noise bands on the display during initial focus window launch
+        if self.settings_profile == "Focus":
+            self.glWidget.force_clear = True
+
     def keyPressEvent(self, event):
         match event.key():
             case Qt.Key.Key_Escape:
@@ -477,6 +474,11 @@ class MainWindow(QMainWindow):
                 else:
                     self.glWidget.focused_uri = None
                     self.collapseSplitter()
+            case Qt.Key.Key_D:
+                if event.modifiers() & Qt.KeyboardModifier.ControlModifier:
+                    self.cameraPanel.btnDiscoverClicked()
+                    self.cameraPanel.lstCamera.setFocus()
+
         super().keyPressEvent(event)
 
     def showEvent(self, event):
@@ -493,6 +495,14 @@ class MainWindow(QMainWindow):
         if self.settingsPanel.discover.chkAutoDiscover.isChecked():
             self.cameraPanel.btnDiscoverClicked()
 
+        # this will clear display artifacts on launch
+        self.glWidget.last_player_count = 1
+
+        if self.settings_profile == "Reader":
+            self.picturePanel.tree.setFocus()
+        elif self.settings_profile == "gui":
+            self.cameraPanel.lstCamera.setFocus()
+
         super().showEvent(event)
 
     def resizeEvent(self, event):
@@ -503,52 +513,41 @@ class MainWindow(QMainWindow):
         super().resizeEvent(event)
 
     def startAllCameras(self):
+        lstCamera = self.cameraPanel.lstCamera
+        if not lstCamera: return
         try:
-            lstCamera = self.cameraPanel.lstCamera
-            if lstCamera:
-                cameras = [lstCamera.item(x) for x in range(lstCamera.count())]
-                for camera in cameras:
-                    self.cameraPanel.setCurrentCamera(camera.uri())
-                    self.cameraPanel.onItemDoubleClicked(camera)
+            cameras = [lstCamera.item(x) for x in range(lstCamera.count())]
+            for camera in cameras:
+                self.cameraPanel.setCurrentCamera(camera.uri())
+                self.cameraPanel.onItemDoubleClicked(camera)
         except Exception as ex:
             logger.error(f'Start all cameras error : {ex}')
 
+    def closeAnyPlayingFiles(self):
+        self.pm.lock()
+        for player in self.pm.players.values():
+            if not player.isCameraStream():
+                player.requestShutdown()
+        self.pm.unlock()
+
     def closeAllStreams(self):
         try:
             for timer in self.timers.values():
                 self.signals.stopReconnect.emit(timer.uri)
                 timer.stop()
-            for player in self.pm.players:
-                player.requestShutdown()
 
-            self.pm.auto_start_mode = False
+            self.pm.lock()
+            for player in self.pm.players.values():
+                player.requestShutdown()
+            self.pm.unlock()
+
             lstCamera = self.cameraPanel.lstCamera
             if lstCamera:
                 cameras = [lstCamera.item(x) for x in range(lstCamera.count())]
                 for camera in cameras:
                     camera.setIconIdle()
 
-            count = 0
-            while len(self.pm.players):
-                sleep(0.1)
-                count += 1
-                if count > 50:
-                    logger.error("not all players closed within the allotted time")
-                    for player in self.pm.players:
-                        name = ""
-                        if player.isCameraStream():
-                            name = self.getCameraName(player.uri)
-                        else:
-                            name = player.uri
-                            # seems to be ok for files
-                            self.pm.removePlayer(player.uri)
-                        logger.debug(f'{name} failed orderly shutdown')
-                        # sleep(1)
-                        # This can cause crashing
-                        # self.pm.removePlayer(player.uri)
-                        # logger.error(f'{name} was removed from the list after failing orderly shutdown')
-                    break
-
+            self.pm.waitForAllPlayersClosed()
             self.pm.ordinals.clear()
             self.pm.sizes.clear()
 
@@ -558,6 +557,7 @@ class MainWindow(QMainWindow):
                 if self.settingsPanel:
                     if self.settingsPanel.general:
                         self.settingsPanel.general.btnCloseAll.setText("Start All")
+
         except Exception as ex:
             logger.error(f'Close all streams error : {ex}')
 
@@ -566,19 +566,20 @@ class MainWindow(QMainWindow):
             self.closing = True
             self.closeAllStreams()
             self.stopProxyServer()
+            self.stopHttpServer()
             self.stopOnvifServer()
 
             self.settings.setValue(self.geometryKey, self.geometry())
-            super().closeEvent(event)
 
             if self.focus_window:
                 self.focus_window.close()
+                sleep(0.1)
 
             for window in self.external_windows:
                 window.close()
+                sleep(0.1)
 
-            if self.filePanel.control.dlgPicture.isVisible():
-                self.filePanel.control.dlgPicture.close()
+            super().closeEvent(event)
 
         except Exception as ex:
             logger.error(f'Window close error : {ex}')
@@ -590,20 +591,33 @@ class MainWindow(QMainWindow):
     def hideWaitDialog(self):
         self.dlgWait.hide()
 
-    def initializeFocusWindowSettings(self):
-        proxy = None
-        match self.settingsPanel.proxy.proxyType:
-            case ProxyType.CLIENT:
-                proxy = self.settingsPanel.proxy.txtRemote.text()
-            case ProxyType.SERVER:
-                proxy = self.settingsPanel.proxy.lblServer.text().split()[0]
-        focus_settings = QSettings("onvif-gui", "Focus")
-        focus_settings.setValue("settings/proxyType", ProxyType.CLIENT)
-        focus_settings.setValue("settings/proxyRemote", proxy)
-        focus_settings.setValue("settings/autoDiscover", 1)
-        focus_settings.setValue(self.collapsedKey, 1)
-        self.focus_window = onvif_gui.main.MainWindow(settings_profile="Focus")
-        self.focus_window.audioStatus = self.audioStatus
+    def openFocusWindow(self):
+        if not self.focus_window:
+            proxy = None
+            match self.settingsPanel.proxy.proxyType:
+                case ProxyType.CLIENT:
+                    proxy = self.settingsPanel.proxy.txtRemote.text()
+                case ProxyType.SERVER:
+                    proxy = self.settingsPanel.proxy.lblServer.text().split()[0]
+            focus_settings = QSettings("onvif-gui", "Focus")
+            focus_settings.setValue("settings/proxyType", ProxyType.CLIENT)
+            focus_settings.setValue("settings/proxyRemote", proxy)
+            focus_settings.setValue("settings/autoDiscover", 1)
+
+            # some cameras have problem streams that force the use of alternate choices for
+            # display and record profiles, force the focus window to use those settings
+            keys = self.settings.allKeys()
+            for key in keys:
+                parts = key.split("/")
+                if len(parts) > 1:
+                    if parts[1] == "RecordProfile":
+                        value = self.settings.value(key)
+                        camera = parts[0]
+                        focus_key = f'{camera}/DisplayProfile'
+                        focus_settings.setValue(focus_key, value)
+                        focus_key = f'{camera}/RecordProfile'
+                        focus_settings.setValue(focus_key, value)
+            self.focus_window = onvif_gui.main.MainWindow(settings_profile="Focus")
         self.focus_window.show()
 
     def mediaPlayingStarted(self, uri):
@@ -621,20 +635,10 @@ class MainWindow(QMainWindow):
                     if self.settings_profile == "gui":
                         window_name = "Main"
 
-                    logger.debug(f'Camera stream opened {self.getCameraName(uri)}, stream_uri : {profile.stream_uri()}, resolution : {profile.width()} x {profile.height()}, fps: {profile.frame_rate()}, {profile_type}, Window: {window_name}')
-
-                if self.pm.auto_start_mode:
-                    finished = True
-                    cameras = [self.cameraPanel.lstCamera.item(x) for x in range(self.cameraPanel.lstCamera.count())]
-                    for camera in cameras:
-                        state = camera.getStreamState(camera.displayProfileIndex())
-                        if state == StreamState.IDLE:
-                            finished = False
-                    if finished:
-                        self.pm.auto_start_mode = False
+                    if self.settings_profile == "gui":
+                        logger.debug(f'Camera stream opened {self.getCameraName(uri)}, stream_uri : {profile.stream_uri()}, serial_number : {profile.serial_number()}, resolution : {profile.width()} x {profile.height()}, fps: {profile.frame_rate()}, {profile_type}, Window: {window_name}')
 
                 if player := self.pm.getPlayer(uri):
-                    player.clearCache()
                     if player.systemTabSettings():
                         if player.systemTabSettings().record_enable and player.systemTabSettings().record_always:
                             if camera := self.cameraPanel.getCamera(uri):
@@ -646,33 +650,24 @@ class MainWindow(QMainWindow):
                                     if camera.profiles[camera.displayProfileIndex()].uri() == uri:
                                         record = True
                                 if record:
-                                    d = self.settingsPanel.storage.dirArchive.txtDirectory.text()
-                                    if self.settingsPanel.storage.chkManageDiskUsage.isChecked():
-                                        self.diskManager.manageDirectory(d)
-                                    else:
-                                        self.settingsPanel.storage.signals.updateDiskUsage.emit()
-                                    if filename := player.getPipeOutFilename():
+                                    if filename := player.getOutputFilename():
                                         player.toggleRecording(filename)
 
             self.signals.stopReconnect.emit(uri)
             self.signals.started.emit(uri)
+
         except Exception as ex:
             logger.error(f'Exception occured during callback media playing started: {ex}')
 
-    def stopReconnectTimer(self, uri):
-        if timer := self.timers.get(uri, None):
-            timer.lock()
-            timer.stop()
-            timer.unlock()
-
     def mediaPlayingStopped(self, uri):
         try:
             if player := self.pm.getPlayer(uri):
+                request_reconnect = player.request_reconnect
                 self.pm.removePlayer(uri)
-
-                if player.request_reconnect:
+        
+                if request_reconnect:
                     showMessage = True
-                    if timer := self.timers.get(player.uri):
+                    if timer := self.timers.get(uri):
                         if timer.attempting_reconnect:
                             showMessage = False
                     if showMessage:
@@ -680,10 +675,13 @@ class MainWindow(QMainWindow):
                     self.signals.reconnect.emit(uri)
                 else:
                     if self.isCameraStreamURI(uri):
-                        logger.debug(f'Stream closed {self.getCameraName(uri)}')
+                        if self.settings_profile == "gui":
+                            logger.debug(f'Stream closed {self.getCameraName(uri)}')
+
 
                 if self.signals:
                     self.signals.stopped.emit(uri)
+
         except Exception as ex:
             logger.error(f'Exception occurred during callback media playing stopped: {ex}')
 
@@ -694,6 +692,19 @@ class MainWindow(QMainWindow):
             self.timers[uri] = Timer(self, uri)
         self.cameraPanel.syncGUI()
 
+    def stopReconnectTimer(self, uri):
+        if timer := self.timers.get(uri, None):
+            timer.lock()
+            timer.stop()
+            timer.unlock()
+
+    def getActiveTimerCount(self):
+        count = 0
+        for timer in self.timers.values():
+            if timer.isActive():
+                count += 1
+        return count
+
     def infoCallback(self, msg, uri):
         try:
             if msg == "player audio disabled":
@@ -727,12 +738,8 @@ class MainWindow(QMainWindow):
                 logger.error(f'{name}, Message: {msg}')
                 return
 
-            if msg.startswith("SDL_OpenAudioDevice exception"):
-                if profile := self.cameraPanel.getProfile(uri):
-                    profile.setDisableAudio(True)
-                    self.cameraPanel.tabVideo.syncGUI()
-                    self.signals.error.emit("Error: Audio output device initialization has failed, audio for this stream has been disabled")
-                    return
+            if msg.startswith("Reader terminated by timeout"):
+                logger.error(f'{name}, Message: {msg}')
 
             if msg.startswith("Using SDL audio driver"):
                 logger.debug(msg)
@@ -748,11 +755,16 @@ class MainWindow(QMainWindow):
                 camera_name = ""
                 last_msg = ""
 
+                if "SDL_OpenAudioDevice error" in msg:
+                    if profile := self.cameraPanel.getProfile(uri):
+                        profile.setDisableAudio(True)
+                        player.disable_audio = True
+                        self.cameraPanel.tabVideo.syncGUI()
+
                 if camera := self.cameraPanel.getCamera(uri):
                     camera_name = self.getCameraName(uri)
                     last_msg = camera.last_msg
                     camera.last_msg = msg
-
                     self.signals.reconnect.emit(uri)
 
                 if msg != last_msg:
@@ -799,23 +811,6 @@ class MainWindow(QMainWindow):
         except Exception as ex:
             logger.error(f'Packet drop error: {ex}')
 
-    def getAudioStatus(self):
-        return self.audioStatus
-    
-    def setAudioStatus(self, status):
-        try:
-            self.audioStatus = status
-            if self.parent_window:
-                self.parent_window.audioStatus = status
-            if self.focus_window:
-                self.focus_window.audioStatus = status
-            if self.reader_window:
-                self.reader_window.audioStatus = status
-            for window in self.external_windows:
-                window.audioStatus = status
-        except Exception as ex:
-            logger.error(f'set audio status exception: {ex}')
-
     def onError(self, msg):
         if not self.closing:
             msgBox = QMessageBox(self)
@@ -946,7 +941,8 @@ class MainWindow(QMainWindow):
     
     def initializeClient(self, ip_addr):
         try:
-            self.client = kankakee.Client(f'{ip_addr}:8550')
+            self.client = kankakee.Client(ip_addr, 8550)
+            self.client.timeout = self.settingsPanel.proxy.spnTimeout.value()
             self.client.clientCallback = self.clientProtocols.callback
             self.client.errorCallback = self.clientProtocols.error
         except Exception as ex:
@@ -984,14 +980,15 @@ class MainWindow(QMainWindow):
         
     def downloadProxyServer(self, dir):
 
+        version = "v1.15.5"
         hashes = {
-            "mediamtx_v1.12.2_darwin_amd64.tar.gz": "572a766870f821196ec0977fda7993ac5a8c45ba34174b3a048f000e3fe1dd0b",
-            "mediamtx_v1.12.2_darwin_arm64.tar.gz": "df388cb70bcefe3822a63eb884576191120e63099d1fac4314d63d38b60eb238",
-            "mediamtx_v1.12.2_linux_amd64.tar.gz": "f0ec6e21c3cde41d02f186f58636f0ea8ee67c9d44dacf5b9391e85600f56e74",
-            "mediamtx_v1.12.2_linux_arm64.tar.gz": "35803953e27a7b242efb1f25b4d48e3cc24999bcb43f6895383a85d6f8000651",
-            "mediamtx_v1.12.2_linux_armv6.tar.gz": "765156e430b6664d1092116e33c5ba5c5fc711d0fe4a0e5805326852d0fa7523",
-            "mediamtx_v1.12.2_linux_armv7.tar.gz": "b10a5267113bc013339e3bfc7b60a3c32aeba1bf56f0e86be36f02b123ff1328",
-            "mediamtx_v1.12.2_windows_amd64.zip": "f83b9954f3b39f2aed5e93dd739ce2e3dbb276aa21c1759547ba6d858ca68671"
+            f"mediamtx_{version}_darwin_amd64.tar.gz": "6418cf6df9f00b04f3814c54a35d8f8a5e6625a187b5b51622feb070b71527ac",
+            f"mediamtx_{version}_darwin_arm64.tar.gz": "116150e6900ed2ae845cf5113fab8007ad639a36d80c1a09c56e091ddcc5b907",
+            f"mediamtx_{version}_linux_amd64.tar.gz": "158ebff59739d723a5bb60a7c6c42d908d378609f6237a94e583edd5456f7322",
+            f"mediamtx_{version}_linux_arm64.tar.gz": "515f08cb99add42e66fc6f897f19942a3ee319bb001ef509e6e7ffdd5031f162",
+            f"mediamtx_{version}_linux_armv6.tar.gz": "d59047e71a43800ea5c986035d5e63b84c65935c478a22399a513573d0d6d53c",
+            f"mediamtx_{version}_linux_armv7.tar.gz": "4a47a4208cb68216a98b6b9bc40ce5925455276dedc63c1fb6834b4fa6f947f1",
+            f"mediamtx_{version}_windows_amd64.zip": "19cd9d1fbb76225380859109175b7547d2e68b4b70858be4fa565604743acf8d"
         }
 
         try:
@@ -1020,7 +1017,6 @@ class MainWindow(QMainWindow):
                 case "win32":
                     operating_system = "windows"
 
-            version = "v1.12.2"
             home = "https://github.com/bluenviron/mediamtx/releases/download"
             suffix = "tar.gz"
             if operating_system == "windows":
@@ -1073,7 +1069,7 @@ class MainWindow(QMainWindow):
                         contents = file.read()
 
                     contents = contents.replace("srt: yes", "srt: no")
-                    contents = contents.replace("webrtc: yes", "webrtc: no")
+                    #contents = contents.replace("webrtc: yes", "webrtc: no")
                     contents = contents.replace("hls: yes", "hls: no")
                     contents = contents.replace("rtmp: yes", "rtmp: no")
 
@@ -1123,6 +1119,31 @@ class MainWindow(QMainWindow):
             self.mediamtx_process = None
             logger.debug("Proxy server stopped")
 
+    def startHttpServer(self):
+        try:
+            if not self.http_process:
+                self.http_process = subprocess.Popen([sys.executable, f'{os.path.dirname(os.path.realpath(__file__))}/server.py'], env=os.environ.copy(), start_new_session=True)
+                return_code = self.http_process.returncode
+                print("RC", return_code)
+                logger.debug("starting http server from dir", os.path.dirname(os.path.realpath(__file__)))
+        except Exception as ex:
+            logger.error(f'Error starting http server: {ex}')
+            self.signals.error.emit(f'Error starting http server: {ex}')
+
+    def stopHttpServer(self):
+        if self.settings_profile == "Reader":
+            return
+        try:
+            requests.post("http://127.0.0.1:8800/shutdown", timeout=1)
+        except Exception:
+            pass
+
+        if self.http_process:
+            self.http_process.terminate()
+            self.http_process.wait(timeout=5)
+            self.http_process = None
+            logger.debug("Http server stopped")
+
     def getProxyURI(self, arg):
         return self.proxies.get(arg, arg)
     
@@ -1249,67 +1270,71 @@ class MainWindow(QMainWindow):
             return output + error_output
         
 def run():
-    clear_settings = False
+    parser = argparse.ArgumentParser(description="onvif-gui")
+    parser.add_argument("-p", "--profile", type=str, default="gui")
+    parser.add_argument("-c", "--clear", action="store_true")
+    parser.add_argument("-i", "--icon", action="store_true")
+    args = parser.parse_args()
 
-    if len(sys.argv) > 1:
-        if str(sys.argv[1]) == "--clear":
-            clear_settings = True
+    
+    clear_settings = args.clear
+    settings_profile = args.profile
 
-        if str(sys.argv[1]) == "--icon":
-            if sys.platform == "win32":
-                filename = os.environ['HOMEPATH'] + "/.cache/onvif-gui/logs/logs.txt"
-            else:
-                filename = os.environ['HOME'] + "/.cache/onvif-gui/logs/logs.txt"
-            logger.add(filename, rotation="1 MB")
+    if args.icon:
+        if sys.platform == "win32":
+            filename = os.environ['HOMEPATH'] + "/.cache/onvif-gui/logs/logs.txt"
+        else:
+            filename = os.environ['HOME'] + "/.cache/onvif-gui/logs/logs.txt"
+        logger.add(filename, rotation="1 MB")
 
-            if sys.platform == "win32":
-                icon = f'{os.path.split(__file__)[0]}\\resources\\onvif-gui.ico'
-                working_dir = f'{os.path.split(sys.executable)[0]}'
-                if "conda" in working_dir:
-                    working_dir = f'{working_dir}\\Scripts'
-                executable = f'{working_dir}\\onvif-gui.exe'
-                try:
-                    import winshell
-                    link_filepath = f'{Path(winshell.desktop())}\\Onvif GUI.lnk'
-                    with winshell.shortcut(link_filepath) as link:
-                        link.path = executable
-                        link.description = "Onvif GUI"
-                        link.arguments = ""
-                        link.icon_location = (icon, 0)
-                        link.working_directory = working_dir
+        if sys.platform == "win32":
+            icon = f'{os.path.split(__file__)[0]}\\resources\\onvif-gui.ico'
+            working_dir = f'{os.path.split(sys.executable)[0]}'
+            if "conda" in working_dir:
+                working_dir = f'{working_dir}\\Scripts'
+            executable = f'{working_dir}\\onvif-gui.exe'
+            try:
+                import winshell
+                link_filepath = f'{Path(winshell.desktop())}\\Onvif GUI.lnk'
+                with winshell.shortcut(link_filepath) as link:
+                    link.path = executable
+                    link.description = "Onvif GUI"
+                    link.arguments = ""
+                    link.icon_location = (icon, 0)
+                    link.working_directory = working_dir
 
-                    logger.debug(f'Desktop icon created for executable {executable}')
-                except Exception as ex:
-                    logger.debug(f'Error attempting to create desktop icon : {ex}')
-            else:
-                icon = f'{os.path.split(__file__)[0]}/resources/onvif-gui.png'
-                executable = f'{os.path.split(sys.executable)[0]}/onvif-gui %U'
+                logger.debug(f'Desktop icon created for executable {executable}')
+            except Exception as ex:
+                logger.debug(f'Error attempting to create desktop icon : {ex}')
+        else:
+            icon = f'{os.path.split(__file__)[0]}/resources/onvif-gui.png'
+            executable = f'{os.path.split(sys.executable)[0]}/onvif-gui %U'
 
-                contents = (f'[Desktop Entry]\n'
-                            f'Version={VERSION}\n'
-                            f'Name=Onvif GUI\n'
-                            f'Comment=IP Camera Application\n'
-                            f'Exec={executable}\n'
-                            f'Terminal=false\n'
-                            f'Icon={icon}\n'
-                            f'StartupWMClass=python3\n'
-                            f'Type=Application\n'
-                            f'Categories=Utility\n')
-                
-                try:
-                    with open('/usr/share/applications/onvif-gui.desktop', 'w') as f:
-                        f.write(contents)
-                    #logger.debug("Desktop icon created successfully")
-                    print("Desktop Icon Created Successfully, look in the system Applications to find Onvif GUI\n")
-                except Exception as ex:
-                    logger.error(f'Error attempting to create desktop icon : {ex}')
+            contents = (f'[Desktop Entry]\n'
+                        f'Version={VERSION}\n'
+                        f'Name=Onvif GUI\n'
+                        f'Comment=IP Camera Application\n'
+                        f'Exec={executable}\n'
+                        f'Terminal=false\n'
+                        f'Icon={icon}\n'
+                        f'StartupWMClass=python3\n'
+                        f'Type=Application\n'
+                        f'Categories=Utility\n')
+            
+            try:
+                with open('/usr/share/applications/onvif-gui.desktop', 'w') as f:
+                    f.write(contents)
+                #logger.debug("Desktop icon created successfully")
+                print("Desktop Icon Created Successfully, look in the system Applications to find Onvif GUI\n")
+            except Exception as ex:
+                logger.error(f'Error attempting to create desktop icon : {ex}')
 
-            sys.exit()
+        sys.exit()
 
     app = QApplication(sys.argv)
 
     app.setStyle('Fusion')
-    window = MainWindow(clear_settings)
+    window = MainWindow(clear_settings=clear_settings, settings_profile=settings_profile)
     window.show()
     app.exec()
 
