@@ -4,4 +4,6 @@ from .imagetab import ImageTab
 from .systemtab import SystemTab
 from .logindialog import LoginDialog
 from .ptztab import PTZTab
-from .datastructures import Session, Camera
\ No newline at end of file
+from .datastructures import Session, Camera
+from .cameralist import CameraList
+from .snapshot import Snapshot
\ No newline at end of file
