@@ -1,5 +1,5 @@
 #/********************************************************************
-# libonvif/onvif-gui/onvif_gui/panels/settings/proxy.py 
+# onvif-gui/onvif_gui/panels/settings/proxy.py 
 #
 # Copyright (c) 2024  Stephen Rhodes
 #
@@ -19,7 +19,7 @@
 
 from PyQt6.QtWidgets import QMessageBox, QLineEdit, QPushButton, \
     QGridLayout, QWidget, QLabel, QMessageBox, QRadioButton, \
-    QGroupBox, QCheckBox, QComboBox, QDialog, QDialogButtonBox
+    QGroupBox, QCheckBox, QComboBox, QSpinBox
 from PyQt6.QtCore import Qt
 from onvif_gui.components import DirectorySelector
 from onvif_gui.enums import ProxyType
@@ -28,6 +28,7 @@ import libonvif as onvif
 import ipaddress
 import os
 import sys
+import html
 
 class ProxyOptions(QWidget):
     def __init__(self, mw):
@@ -41,6 +42,8 @@ class ProxyOptions(QWidget):
         self.broadcastInterfaceKey = "settings/proxyBroadcastInterface"
         self.broadcastEnableKey = "setings/enableBroadcasting"
         self.broadcastAllInterfacesKey = "settings/broadcastAllInterfaces"
+        self.timeoutKey = "settings/clientSocketTimeout"
+        self.httpKey = "settings/enableHttpServer"
 
         self.if_addrs = []
         session = onvif.Session()
@@ -68,10 +71,15 @@ class ProxyOptions(QWidget):
         self.txtDirectoryMTX = DirectorySelector(mw, "MTXDir", "Dir", self.getProxyServerDir())
         self.txtDirectoryMTX.setEnabled(not self.chkAutoDownload.isChecked())
 
+        self.cmbProxyLog = QComboBox()
+        self.cmbProxyLog.addItems(["error", "warn", "info", "debug"])
+
         self.grpMediaMTX = QGroupBox("Media MTX Settings")
         lytMediaMTX = QGridLayout(self.grpMediaMTX)
-        lytMediaMTX.addWidget(self.chkAutoDownload,       0, 0, 1, 1)
-        lytMediaMTX.addWidget(self.txtDirectoryMTX,       1, 0, 1, 1)
+        lytMediaMTX.addWidget(self.chkAutoDownload,       0, 0, 1, 2)
+        lytMediaMTX.addWidget(self.txtDirectoryMTX,       1, 0, 1, 2)
+        lytMediaMTX.addWidget(QLabel("Log Level"),        2, 0, 1, 1, Qt.AlignmentFlag.AlignRight)
+        lytMediaMTX.addWidget(self.cmbProxyLog,           2, 1, 1, 1)
 
         self.cmbInterfaces = QComboBox()
         self.lblInterfaces = QLabel("Network")
@@ -116,6 +124,21 @@ class ProxyOptions(QWidget):
         self.chkListen.stateChanged.connect(self.chkListenChecked)
         self.chkListen.setEnabled(False)
 
+        self.spnTimeout = QSpinBox()
+        self.spnTimeout.setValue(int(self.mw.settings.value(self.timeoutKey, 5)))
+        self.spnTimeout.valueChanged.connect(self.spnTimeoutChanged)
+
+        self.lblTimeout = QLabel("Client socket timeout (seconds)")
+        pnlTimeout = QWidget()
+        lytTimeout = QGridLayout(pnlTimeout)
+        lytTimeout.addWidget(self.lblTimeout,  0, 0, 1, 1)
+        lytTimeout.addWidget(self.spnTimeout,  0, 1, 1, 1, Qt.AlignmentFlag.AlignCenter)
+        lytTimeout.setContentsMargins(0, 0, 0, 0)
+
+        self.chkEnableHttpServer = QCheckBox("Enable HTTP server")
+        self.chkEnableHttpServer.setChecked(int(mw.settings.value(self.httpKey, 0)))
+        self.chkEnableHttpServer.stateChanged.connect(self.chkEnableHttpServerChecked)
+
         self.proxyType = None
         match self.mw.settings.value(self.proxyTypeKey, ProxyType.STAND_ALONE):
             case ProxyType.STAND_ALONE:
@@ -135,6 +158,21 @@ class ProxyOptions(QWidget):
         if self.mw.settings_profile != "gui":
             self.radServer.setEnabled(False)
 
+        tag = "info"
+        if self.proxyType == ProxyType.SERVER:
+            try:
+                filename = os.path.join(self.getProxyServerDir(), "mediamtx.yml") 
+                with open(filename, 'r') as file:
+                    content = file.read()
+                    key = "logLevel:"
+                    index = content.index(key)
+                    eol = content.index("\n", index)
+                    tag = content[index + len(key):eol].strip()
+            except Exception as ex:
+                logger.error(f"Media MTX configuration file read error: {ex}")
+        self.cmbProxyLog.setCurrentText(tag)
+        self.cmbProxyLog.currentTextChanged.connect(self.cmbProxyLogChanged)
+
         lytProxyType = QGridLayout(self.grpProxyType)
         lytProxyType.addWidget(self.radStandAlone,       0, 0, 1, 1)
         lytProxyType.addWidget(QLabel(),                 1, 0, 1, 2)
@@ -143,13 +181,18 @@ class ProxyOptions(QWidget):
         lytProxyType.addWidget(self.btnUpdate,           3, 0, 1, 1, Qt.AlignmentFlag.AlignCenter)
         lytProxyType.addWidget(self.txtRemote,           3, 1, 1, 1)
         lytProxyType.addWidget(self.chkListen,           4, 0, 1, 2)
-        lytProxyType.addWidget(QLabel(),                 6, 0, 1, 1)
+        lytProxyType.addWidget(pnlTimeout,          5, 0, 1, 2)
+        #lytProxyType.addWidget(QLabel(),                 6, 0, 1, 1)
         lytProxyType.addWidget(self.radServer,           7, 0, 1, 1)
         lytProxyType.addWidget(self.lblConnect,          7, 1, 1, 1)
         lytProxyType.addWidget(self.lblServer,           8, 0, 1, 2, Qt.AlignmentFlag.AlignCenter)
-        lytProxyType.addWidget(self.grpMediaMTX,         9, 0, 1, 2)
-        lytProxyType.addWidget(self.grpAlarmBroadcast,  10, 0, 1, 2)
-        lytProxyType.addWidget(QLabel(),                11, 0, 1, 1)
+
+        lytProxyType.addWidget(self.chkEnableHttpServer,  11, 0, 1, 1)
+
+        lytProxyType.addWidget(self.grpMediaMTX,         12, 0, 1, 2)
+
+        lytProxyType.addWidget(self.grpAlarmBroadcast,  13, 0, 1, 2)
+        lytProxyType.addWidget(QLabel(),                15, 0, 1, 1)
 
         lytMain = QGridLayout(self)
         lytMain.addWidget(self.grpProxyType,  0, 0, 1, 2)
@@ -169,7 +212,7 @@ class ProxyOptions(QWidget):
     def setProxyType(self, type):
         if type == self.proxyType:
             return
-        if len(self.mw.pm.players):
+        if self.mw.pm.countPlayers():
             ret = QMessageBox.question(self.mw, "Closing Streams", "Continuing with this operation will close all open streams and may require the application to restart.\n\nAre you sure you want to do this?")
             if ret == QMessageBox.StandardButton.No:
                 match (self.proxyType):
@@ -186,14 +229,18 @@ class ProxyOptions(QWidget):
         self.mw.settings.setValue(self.proxyTypeKey, type)
         self.mw.stopProxyServer()
         self.mw.stopOnvifServer()
+        self.mw.stopHttpServer()
 
         self.grpAlarmBroadcast.setEnabled(type == ProxyType.SERVER)
         self.grpMediaMTX.setEnabled(type == ProxyType.SERVER)
+        self.chkEnableHttpServer.setEnabled(type == ProxyType.SERVER)
 
         if type == ProxyType.SERVER and self.mw.settings_profile == "gui":
             self.setServersLabel()
             self.mw.manageBroadcaster(self.getInterfaces())
             self.mw.startProxyServer(self.chkAutoDownload.isChecked(), self.getProxyServerDir())
+            if self.chkEnableHttpServer.isChecked():
+                self.mw.startHttpServer()
             self.mw.startOnvifServer("")
 
         if not hasattr(self.mw, "cameraPanel"):
@@ -269,7 +316,7 @@ class ProxyOptions(QWidget):
 
     def btnUpdateClicked(self):
         try:
-            if len(self.mw.pm.players):
+            if self.mw.pm.countPlayers():
                 QMessageBox.information(self.mw, "Closing Streams", "All current streams will be closed")
                 self.mw.closeAllStreams()
 
@@ -277,11 +324,10 @@ class ProxyOptions(QWidget):
             comps = arg[:len(arg)-1][7:].split(":")
             ip_addr = comps[0]
             port = 8550
-            remote = f'{ip_addr}:{port}'
             if not self.mw.client:
-                self.mw.initializeClient(remote)
+                self.mw.initializeClient(ip_addr)
             else:
-                self.mw.client.setEndpoint(remote)
+                self.mw.client.setEndpoint(ip_addr, port)
             self.proxyRemote = self.txtRemote.text()
 
             if lstCamera := self.mw.cameraPanel.lstCamera:
@@ -303,6 +349,19 @@ class ProxyOptions(QWidget):
         self.mw.settings.setValue(self.listenKey, state)
         self.manageAnalyzers(state)
 
+    def chkEnableHttpServerChecked(self, state):
+        self.mw.settings.setValue(self.httpKey, state)
+        if state:
+            self.createMtxIndex()
+            self.mw.startHttpServer()
+        else:
+            self.mw.stopHttpServer()
+
+    def spnTimeoutChanged(self, value):
+        print("spnTimeoutChanged", value)
+        self.mw.settings.setValue(self.timeoutKey, value)
+        self.mw.client.timeout = value
+
     def grpAlarmBroadcastClicked(self):
         self.mw.settings.setValue(self.broadcastEnableKey, int(self.grpAlarmBroadcast.isChecked()))
         self.mw.manageBroadcaster(self.getInterfaces())
@@ -317,10 +376,17 @@ class ProxyOptions(QWidget):
         else:
             return True
         
+    def getTabIndexByName(self, name):
+        index = -1
+        for i in range(self.mw.tab.count()):
+            if self.mw.tab.tabText(i) == name:
+                return i
+        return index
+        
     def manageAnalyzers(self, state):
         if state:
-            self.mw.tab.removeTab(4)
-            self.mw.tab.removeTab(3)
+            self.mw.tab.removeTab(self.getTabIndexByName("Video"))
+            self.mw.tab.removeTab(self.getTabIndexByName("Audio"))
             self.mw.videoConfigure = None
             self.mw.audioConfigure = None
             self.mw.startListener(self.getInterfaces())
@@ -342,17 +408,28 @@ class ProxyOptions(QWidget):
         match self.proxyType:
             case ProxyType.SERVER:
                 if self.grpAlarmBroadcast.isChecked():
-                    # only able to get this to work on one interface at a time needs research
-                    '''
-                    if self.chkBroadcastAllInterfaces.isChecked():
-                        if_addrs = self.if_addrs
-                    else:
-                        if_addrs.append(self.cmbInterfaces.currentText())
-                    '''
                     if_addrs.append(self.cmbInterfaces.currentText())
             case ProxyType.CLIENT:
                 if_addrs = self.if_addrs
         return if_addrs
+    
+    def cmbProxyLogChanged(self, arg):
+        try:
+            filename = os.path.join(self.getProxyServerDir(), "mediamtx.yml") 
+            with open(filename, 'r') as file:
+                content = file.read()
+
+            key = "logLevel:"
+            index = content.index(key)
+            eol = content.index("\n", index)
+            tag = content[index:eol].strip()
+            content = content.replace(tag, f'logLevel: {arg}')
+
+            with open(filename, 'w') as file:
+                file.write(content) 
+
+        except Exception as ex:
+            logger.error(f"Media MTX configuration file read error: {ex}")
         
     def setMediaMTXProxies(self):
         try:
@@ -429,14 +506,70 @@ class ProxyOptions(QWidget):
                         lines.append(readline)
                         relevant = False
 
-            config = open(config_filename, 'w')
-            for line in lines:
-                config.write(line)
-            config.close()
+            
+            with open(config_filename, 'w') as config:
+                for line in lines:
+                    config.write(line)
+
+            self.createMtxIndex()
 
         except Exception as ex:
             logger.error(f'Error setting Media MTX proxies: {ex}')
             self.mw.signals.error.emit(f'Error setting Media MTX proxies: {ex}')
 
-    def btnTestClicked(self):
-        print("test button clicked")
+    def createMtxIndex(self):
+        try:
+            links = {}
+            lst = self.mw.cameraPanel.lstCamera
+            cameras = [lst.item(x) for x in range(lst.count())]
+            for camera in cameras:
+                #print("camera", camera.name())
+                for profile in camera.profiles:
+                    #print("    profile:", f"{camera.serial_number()}/{profile.profile()}")
+                    links[f"{camera.name()} ({profile.profile()})"] = f"{camera.serial_number()}/{profile.profile()}"
+
+            #for link in links:
+            #    print("Link", link, links[link])
+
+            #print(self.generateIndexHtml(links))
+
+            with open(f'{self.getProxyServerDir()}/index.html', 'w') as file:
+                file.write(self.generateIndexHtml(links))
+
+        except Exception as ex:
+            logger.error(f"error generating MTX index.html: {ex}")
+
+    def generateIndexHtml(self, cameras, webrtc_port=8889):
+        links_html = "\n".join(
+            f'    <li><a class="cam" data-path="{html.escape(path)}">'
+            f'{html.escape(name)}</a></li>'
+            for name, path in cameras.items()
+        )
+
+        return f"""<!DOCTYPE html>
+    <html>
+    <head>
+    <meta charset="utf-8">
+    <title>Cameras</title>
+    </head>
+    <body>
+
+    <h1>Camera List</h1>
+
+    <ul>
+    {links_html}
+    </ul>
+
+    <script>
+    const webrtcPort = {webrtc_port};
+    const loc = window.location;
+
+    document.querySelectorAll(".cam").forEach(a => {{
+        const path = a.dataset.path;
+        a.href = `${{loc.protocol}}//${{loc.hostname}}:${{webrtcPort}}/${{path}}`;
+    }});
+    </script>
+
+    </body>
+    </html>
+    """
