@@ -35,12 +35,14 @@
 #endif
 
 int longopt = 0;
-#define VERSION "1.4.7"
+#define VERSION "1.4.9"
 
 static struct option longopts[] = {
 	{ "user",       required_argument, NULL,      'u'},
 	{ "password",   required_argument, NULL,      'p'},
 	{ "all",        no_argument,       NULL,      'a'},
+	{ "command",    no_argument,       NULL,      'c'},
+	{ "response",   no_argument,       NULL,      'r'},
 	{ "safe_off",   no_argument,       NULL,      's'},
 	{ "help",       required_argument, NULL,      'h'},
 	{ "version",    no_argument,       NULL,      'v'},
@@ -53,7 +55,7 @@ static const char *password = nullptr;
 
 static void usage()
 {
-	std::cout << "Usage: onvif-util [-ahsv] [-u <user>] [-p <password>] [command]" << std::endl;
+	std::cout << "Usage: onvif-util [-ahsvrc] [-u <user>] [-p <password>] [command]" << std::endl;
 }
 
 static void showAll()
@@ -110,6 +112,8 @@ static void showHelp()
 			  << "      -a  poll all cameras on network and reply with host name\n"
 			  << "      -u  username\n"
 			  << "      -p  password\n"
+			  << "      -c  show command sent to camera\n"
+			  << "      -r  show camera response\n"
 			  << "      -s  safe mode off, enable applications for viewer and browser to run\n\n"
 			  << "  To view all cameras on the network:\n"
 			  << "  onvif-util -a\n\n"
@@ -195,6 +199,16 @@ void show(const std::vector<std::string>& args)
 	}
 }
 
+void show_camera_response()
+{
+	setShowCameraResponse(true);
+}
+
+void show_command_sent()
+{
+	setShowCommandSent(true);
+}
+
 void profileCheck(OnvifData* onvif_data, const std::vector<std::string>& args)
 {
 	int index = 0;
@@ -223,7 +237,7 @@ int main(int argc, char **argv)
 	bool time_sync = false;
 
 	int ch;
-	while ((ch = getopt_long(argc, argv, "u:p:ahsvt", longopts, NULL)) != -1) {
+	while ((ch = getopt_long(argc, argv, "u:p:ahsvtrc", longopts, NULL)) != -1) {
 		switch (ch) {
             case 'u':
 				username = optarg;
@@ -234,6 +248,12 @@ int main(int argc, char **argv)
 			case 'a':
 				showAll();
 				exit(0);
+			case 'r':
+				show_camera_response();
+				break;
+			case 'c':
+				show_command_sent();
+				break;
 			case 'h':
 				usage();
 				showHelp();
@@ -297,17 +317,33 @@ int main(int argc, char **argv)
 					found = true;
 					if (username) strcpy(onvif_data->username, username);
 					if (password) strcpy(onvif_data->password, password);
-					if (getDeviceInformation(onvif_data)  == 0) {
-						std::cout << "  successfully connected to host" << "\n";
-						std::cout << "    name:   " << onvif_data->camera_name << "\n";
-						std::cout << "    serial: " << onvif_data->serial_number << "\n" << std::endl;
-			
-						// Initializing the session properly with the camera requires calling getCapabilities
-						if (getCapabilities(onvif_data)) {
-							std::cout << "ERROR: get capabilities - " << onvif_data->last_error << "\n" << std::endl;
+
+					getTimeOffset(onvif_data);
+					time_t initial_offset = onvif_data->time_offset;
+					int count = 1;
+					int direction = 1;
+
+					while (getCapabilities(onvif_data)) {
+						onvif_data->time_offset = initial_offset + 3600 * direction * count;
+						if (direction < 0) count++;
+						direction = ~direction + 1;
+						if (count > 2) {
+							std::stringstream str;
+							str << "Get capabilities error: " << onvif_data->camera_name << " : " << onvif_data->last_error << " camera interrogation failed";
 							exit(1);
 						}
+						else {
+							continue;
+						}
+					}
 
+					if (getDeviceInformation(onvif_data)  == 0) {
+						std::cout << "  successfully connected to host" << "\n"
+						          << "    name:   " << onvif_data->camera_name << "\n"
+						          << "    serial: " << onvif_data->serial_number << "\n"
+						          << "    xaddrs: " << onvif_data->xaddrs << "\n"
+						          << std::endl;
+			
 						if (time_sync) {
 							std::cout << "  Time sync requested" << std::endl;
 							std::vector<std::string> tmp;
@@ -339,34 +375,66 @@ int main(int argc, char **argv)
 
 	if (!found) {
 		std::stringstream xaddrs;
-		xaddrs << "http://" << wanted << ":80/onvif/device_service";
+		size_t position = std::string(wanted).find(":");
+		if (position != std::string::npos) {
+			xaddrs << "http://" << wanted << "/onvif/device_service";
+		}
+		else {
+			xaddrs << "http://" << wanted << ":80/onvif/device_service";
+			std::cout << "\nDid not find camera with onvif, the program will attempt to connect to the device using the default port 80. If you need to use a different port, please add the port to the end of the input IP address using a colon e.g. 10.1.1.1:8000" << std::endl;
+		}
+
+		std::cout << "\nusing xaddrs " << xaddrs.str() << std::endl;
+
 		if (username) strcpy(onvif_data->username, username);
 		if (password) strcpy(onvif_data->password, password);
 		strcpy(onvif_data->xaddrs, xaddrs.str().c_str());
 		strcpy(onvif_data->device_service, xaddrs.str().c_str());
-		if (getCapabilities(onvif_data)) {
-			std::cout << "ERROR: get capabilities - " << onvif_data->last_error << "\n" << std::endl;
-			exit(1);
-		}
-		else {
-			if (getDeviceInformation(onvif_data)  == 0) {
-				std::cout << "  successfully connected to host" << "\n";
-				std::cout << "    name:   " << onvif_data->camera_name << "\n";
-				std::cout << "    serial: " << onvif_data->serial_number << "\n" << std::endl;
-				found = true;
-				if (time_sync) {
-					std::cout << "  Time sync requested" << std::endl;
-					std::vector<std::string> tmp;
-					profileCheck(onvif_data, tmp);
-					if (setSystemDateAndTime(onvif_data)) throw std::runtime_error(cat("set system date and time - ", onvif_data->last_error));
-					std::cout << "  Camera date and time has been synchronized without regard to camera timezone\n" << std::endl;
-					exit(0);
-				}
+
+		char host[128];
+		extractHost(onvif_data->xaddrs, host);
+		getHostname(onvif_data);
+	    extractOnvifService(onvif_data->device_service, true);
+		strcpy(onvif_data->camera_name, host);
+
+		getTimeOffset(onvif_data);
+		time_t initial_offset = onvif_data->time_offset;
+		int count = 1;
+		int direction = 1;
+
+		while (getCapabilities(onvif_data)) {
+			onvif_data->time_offset = initial_offset + 3600 * direction * count;
+			if (direction < 0) count++;
+			direction = ~direction + 1;
+			if (count > 2) {
+				std::stringstream str;
+				str << "Get capabilities error: " << onvif_data->camera_name << " : " << onvif_data->last_error << " camera interrogation failed";
+				std::cout << str.str() << std::endl;
+				exit(1);
 			}
 			else {
-				std::cout << "Failed to connect to camera" << std::endl;
+				continue;
 			}
 		}
+
+		if (getDeviceInformation(onvif_data)  == 0) {
+			std::cout << "\n  successfully connected to host" << "\n";
+			std::cout << "    name:   " << onvif_data->camera_name << "\n";
+			std::cout << "    serial: " << onvif_data->serial_number << "\n";
+			std::cout << "    xaddrs: " << onvif_data->xaddrs << "\n" << std::endl;
+			found = true;
+			if (time_sync) {
+				std::cout << "  Time sync requested" << std::endl;
+				std::vector<std::string> tmp;
+				profileCheck(onvif_data, tmp);
+				if (setSystemDateAndTime(onvif_data)) throw std::runtime_error(cat("set system date and time - ", onvif_data->last_error));
+				std::cout << "  Camera date and time has been synchronized without regard to camera timezone\n" << std::endl;
+				exit(0);
+			}
+		}
+		else {
+			std::cout << "Failed to connect to camera" << std::endl;
+		}
 	}
 
 	if (!found) {
@@ -378,7 +446,7 @@ int main(int argc, char **argv)
 
 	#ifdef _WIN32
 	quit = "quit\r";
-#endif
+	#endif
 
 	std::string cmd;
 	while (cmd != quit) {
@@ -425,7 +493,7 @@ int main(int argc, char **argv)
 					}
 					profileCheck(onvif_data, args);
 					if (getSnapshotUri(onvif_data)) throw std::runtime_error(cat("get snapshot - ", onvif_data->last_error));
-					std::string uri(onvif_data->stream_uri);
+					std::string uri(onvif_data->snapshot_uri);
 					if (add_pass) {
 						uri = snapshot_with_pass(onvif_data);
 					}
@@ -434,8 +502,8 @@ int main(int argc, char **argv)
 				else if (args[0] == "capabilities") {
 					std::cout << "  event_service:   " << onvif_data->event_service << "\n";
 					std::cout << "  imaging_service: " << onvif_data->imaging_service << "\n";
-					std::cout << "  media_service:   " << onvif_data->imaging_service << "\n";
-					std::cout << "  ptz_service:     " << onvif_data->imaging_service << "\n" << std::endl;
+					std::cout << "  media_service:   " << onvif_data->media_service << "\n";
+					std::cout << "  ptz_service:     " << onvif_data->ptz_service << "\n" << std::endl;
 				}
 				else if (args[0] == "profiles") {
 					int index = 0;
